String Version = "AS675 V1.0";
String AutorHB = "DL2MAJ 01/25";
/*
 Low Voltage Durchgangsprüfer AS675, Praxisheft 35
 HW : AS675-V1.0
 
 V0.9 - 29Dez24
 Helmut Berka DL2MAJ
 
 9k6, 8N1
 use Arduino Pro Mini with Atmega328P, 3V3/8MHz
*/
 
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <TFT_ILI9163C.h>   // war <TFT_ILI9163C.h>
#include <EEPROM.h> // Library EEPROM, in IDE integriert

// Color definitions
#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0  
#define WHITE   0xFFFF

/*
TFT-Connections to Arduino Pro Mini

 8-1 LED to D2 via R9
 7-2 SCK to D13
 6-3 SDA to D11
 5-4 A0 to D8
 4-5 RST to D9
 3-6 CS to D10
 2-7 GND to GND
 1-8 VCC to 3V3 
 */
 
// declare variables

float Rx = 0.0;  // berechnet aus Messwert und Schaltung
float Rx_alt = 5678.0; // erforderlich zur Dunkelschaltung des Werts im TFT
float Offset = 0.0;  // V aus EEPROM geholt
int Messwert = 0;  // value from ADC (0 .. 1023), contains result of ADC measurement
const float ADCstep = 3.203; // mV per step bei Uref = VCC = 3.28V
const float Umess = 0.092735; // maximale Spannung am Messobjekt
float Uout = 0.0; // Spannung aus Messwert
const float v = 69.1; // Verstaerkung v
boolean Runde1 = true;  // aktiviert Anzeige beim Start bei offenem Testport

// Misc
int Piezo = 7; // Ausgang zum Piezoelement 
int LEDon = 2;    // backlight-LED of TFT uses D2

// Declare an instance of the ILI9163
TFT_ILI9163C tft = TFT_ILI9163C(10, 8, 9);  // was ...(CS, 8, DC);  


void setup(void)
{ // put your setup code here, to run once:
  pinMode(LEDon, OUTPUT); // declare D2 as output
  digitalWrite(LEDon, LOW); // set D2 to LOW, TFT-Licht ausschalten
  
  // Referenzspannung für ADC
//  analogReference(INTERNAL);
  
  // start serial port
  Serial.begin(9600);
  Serial.println(Version);
  
  pinMode(Piezo, OUTPUT); // declare D7 as output

  // start TFT
  tft.begin();
  tft.clearScreen();
  TFT_LED(true);  // LED am TFT einschalten
  
  // Anzeige : AATiS e.V.
  tft.setCursor(5, 15);
  tft.setTextColor(YELLOW);  
  tft.setTextSize(2);
  tft.println("AATiS e.V.");
  delay(5);    
 
  // Anzeige : Version
  tft.setCursor(3, 50);
//  tft.setTextColor(MAGENTA);  
  tft.setTextSize(2);
  tft.println(Version);
  delay(5);
  
  // Anzeige : Autor 
  tft.setCursor(30, 115);
  tft.setTextSize(1);
  tft.setTextColor(MAGENTA); 
  tft.print(AutorHB);

  tone(Piezo, 4500, 400);
  Serial.println(" Ton erzeugt 0");
  Serial.println();
      
  delay(2500);
  
  tft.clearScreen();

  // fallweise Offset laden oder aktualisieren
  ReadADCWert();
  Rx_alt = 5678.0;  // sorgt für Anzeige bei offenem Testport

  if (Rx < 100)  // aktualisieren
  {
    EEPROMschreiben(); 
  }
  else  // laden
  {
    EEPROMlesen();
  }
  
}


void loop() 
{
  // put your main code here, to run repeatedly:
  ReadADCWert();  
  DispResult();
  delay(250);
}


void ReadADCWert()
{
  float HilfsBruchNenner = 9999.0;  // 0.094 * v
  float RxZaehler = 0.0; // Uout * 1000 / HilfsBruchNenner
  float RxNenner = 1.0;  // 1 - HilfsBruchZaehler / HilfsBruchNenner
  int SummeMesswert = 0;
  
  Serial.println();
  Serial.println("---------- neue Messung -- ");
  Serial.print("Messwert = ");
  ADCSRA = 150;     // enable ADC 
  delay(100);    
  for (int i = 1; i <= 5; i++)
  {  
    Messwert = analogRead(A0);    // read the input pin
  //  Messwert = 1000;  // for debug only !!!
    delay(25);
    SummeMesswert = SummeMesswert + Messwert;
    Serial.print(Messwert);
    Serial.print("   ");
  }
  ADCSRA = 0;     // clear ADIF-flag of ADC 

  Messwert = SummeMesswert / 5;
  Uout = (Messwert + Offset) * ADCstep / 1000;   // [V]
  Serial.println();
  Serial.print("Uout = ");
  Serial.print(Uout);
  Serial.println(" V");
  Serial.println();
  Serial.print("Rx_alt = ");
  Serial.print(Rx_alt);
  Serial.println(" Ohm");
  Serial.println();

//  hier Berechnung des Widerstands
//  HilfsBruchZaehler = Uout / 1000;  // mV -> V
  HilfsBruchNenner = Umess * v; // V
  RxZaehler = Uout * 1000 / HilfsBruchNenner;  // Ohm
  RxNenner = 1 - Uout / HilfsBruchNenner;  // 1

  Rx_alt = Rx;
  Rx = RxZaehler / RxNenner;
  Rx = Rx - 10.0; // Korrektur wg. R6 (10 Ohm)
  if (Rx < 0) Rx = 0;
  if (Messwert > 1022) Rx = 1100.0;

  Serial.print("Rx = ");
  Serial.print(Rx);
  Serial.println(" Ohm");
  Serial.println();
}


void DispResult()
{ int Laenge;
  byte Spalte = 80;
  byte Spalte_alt = 80;

 if (Rx_alt < 9.5) Laenge = 1;  // wg. Rundung <9.5 = 1 Stelle
  else if (Rx_alt < 99.5) Laenge = 2;
  else if (Rx_alt < 999.5) Laenge = 3;
  else Laenge = 4;
  
  switch (Laenge)
  { case 1: Spalte_alt = 80;
      break;
    case 2: Spalte_alt = 60;
      break;
    case 3: Spalte_alt = 40;
      break;
    case 4: Spalte_alt = 20;
      break;
  }

  if (Rx < 9.5) 
  { Laenge = 1;
    tone(Piezo, 4500);  // Dauerton
  }
  else if (Rx < 99.5) 
    { Laenge = 2;
      tone(Piezo, 4500, 100); // Ton nur 100ms
    }
  else if (Rx < 999.5) 
    { Laenge = 3;
      tone(Piezo, 4500, 25); // Ton nur 25ms
      delay(100);
    }
  else 
    { Laenge = 4;
      noTone(Piezo);
    }
  
  switch (Laenge)
  { case 1: Spalte = 80;
      break;
    case 2: Spalte = 60;
      break;
    case 3: Spalte = 40;
      break;
    case 4: Spalte = 20;
      break;
  }

  if ((Rx != Rx_alt) or Runde1) // Runde1 sorgt für Anzeige bei Start und offenem Testport
  { Runde1 = false;
    if (Messwert > 1022)  // Anzeige : Widerstand > 1050 Ohm 
    { tft.setCursor(5, 5);   
      tft.setTextSize(1);
      tft.setTextColor(BLACK);
      tft.print("Widerstand = ");
      tft.setCursor(Spalte_alt, 30); 
      tft.setTextSize(3);
      tft.print(Rx_alt, 0);
      tft.setCursor(5, 5);   
      tft.setTextSize(1);
      tft.setTextColor(RED);
      tft.print("Widerstand");
      tft.setCursor(10, 30);
      tft.setTextSize(3);
      tft.print("> 1050");
      tft.setCursor(35, 65);
//      tft.setTextColor(BLACK);
      tft.print("Ohm");
      delay(5);
    }
    else 
    { tft.setCursor(5, 5);
      tft.setTextSize(1);
      tft.setTextColor(GREEN);
      tft.setCursor(5, 5);
      tft.print("Widerstand = "); 
      tft.setCursor(10, 30); 
      tft.setTextSize(3);
      tft.setTextColor(BLACK);
      tft.print("> 1050");
      tft.setCursor(Spalte_alt, 30); 
      tft.print(Rx_alt, 0);
      tft.setCursor(Spalte, 30); 
      tft.setTextColor(GREEN);
      tft.print(Rx, 0);
      tft.setCursor(35, 65);
//      tft.setTextColor(YELLOW);  
      tft.print("Ohm");
      delay(5);
    }
  }
}


void EEPROMschreiben()  
{
  String Offset_str;  //
  byte Laenge;  // Laenge von R_Ltg
  int adr = 0;  // Startadresse fuer EEPROM-Speicher
  int SummeMesswert = 0;
  
  Serial.println("EEPROMschreiben");
  
//  ReadADCWert();  // aktuellen Messleitungswiderstand messen

  Serial.println();
  Serial.println("---------- neue Messung -- ");
  Serial.print("Messwert = ");
  ADCSRA = 150;     // enable ADC 
  delay(100);    
  for (int i = 1; i <= 5; i++)
  {  
    Messwert = analogRead(A0);    // read the input pin
  //  Messwert = 1000;  // for debug only !!!
    delay(25);
    SummeMesswert = SummeMesswert + Messwert;
    Serial.print(Messwert);
    Serial.print("   ");
  }
  ADCSRA = 0;     // clear ADIF-flag of ADC 

  Offset = SummeMesswert / 5;
  Offset = 19 - Offset; // Soll-ADCWert von 10R = 19
  delay(25);
  Serial.println();
  Serial.print("Offset = ");
  Serial.println(Offset);
  Serial.println();
  
  Offset_str = String(Offset);
  Offset_str.trim();
  Laenge = Offset_str.length();

  if (Offset < 50)  // Messleitungswiderstand in EEPROM programmieren
  {
    EEPROM.write(adr, Laenge);
    adr = adr + 1;

    Serial.print("Länge = ");
    Serial.println(Laenge);
  
    for (byte i=0; i<Laenge; i++)  // Offset_str[] schreiben
    {     
      EEPROM.write(adr, Offset_str[i]);
      adr = adr + 1;
          
      Serial.print("Offset_str = ");
      Serial.print(Offset_str[i]);
      Serial.println();
     
    }
  }
  
  Serial.println("EEPROMschreiben Ende");
}


void EEPROMlesen()
{
  byte Stelle;  // aktuelles Byte
  String Offset_str = "";  //
  byte Laenge = 0;  // Laenge von Offset_str
  int adr = 0;  // Startadresse fuer EEPROM-Speicher

  // Laden des Messleitungswiderstands
  Serial.println("Laden des Offsetwerts");

  // Offsetwert aus EEPROM laden
  Laenge = EEPROM.read(adr);
  adr = adr + 1;

//  Serial.print("Länge = ");
//  Serial.println(Laenge);

  for (byte i=0;i<Laenge;i++)  // Offset_str[] laden
  {     
    Stelle = EEPROM.read(adr);
    adr = adr + 1;     

    Offset_str = Offset_str + char(Stelle);
  }

  Offset = Offset_str.toFloat();

  Serial.print("Offset = ");
  Serial.println(Offset);

  Serial.println("Offsetwert geladen !");  
}


void TFT_LED(boolean EIN)
{
  if (EIN) digitalWrite(LEDon, HIGH); // set D2 to HIGH, TFT-Licht einschalten
  else digitalWrite(LEDon, LOW); // set D2 to low, TFT-Licht ausschalten
}
